@@ -9,15 +9,13 @@ from .contract import (
     DeltaNeutralContract)
 from .objects import (
     BarData, CommissionReport, DepthMktDataDescription, Execution, FamilyCode,
-    HistogramData, HistoricalTick, HistoricalTickBidAsk, HistoricalTickLast,
-    NewsProvider, PriceIncrement, SmartComponent, SoftDollarTier, TagValue,
-    TickAttribBidAsk, TickAttribLast)
+    HistogramData, HistoricalSession, HistoricalTick, HistoricalTickBidAsk,
+    HistoricalTickLast, NewsProvider, PriceIncrement, SmartComponent,
+    SoftDollarTier, TagValue, TickAttribBidAsk, TickAttribLast)
 from .order import Order, OrderComboLeg, OrderCondition, OrderState
 from .util import UNSET_DOUBLE, parseIBDatetime
 from .wrapper import Wrapper
 
-__all__ = ['Decoder']
-
 
 class Decoder:
     """Decode IB messages and invoke corresponding wrapper methods."""
@@ -34,8 +32,7 @@ class Decoder:
                 'orderStatus', [
                     int, str, float, float, float, int, int,
                     float, int, str, float], skip=1),
-            4: self.wrap(
-                'error', [int, int, str]),
+            4: self.errorMsg,
             5: self.openOrder,
             6: self.wrap(
                 'updateAccountValue', [str, str, str, str]),
@@ -160,6 +157,13 @@ class Decoder:
                 'completedOrdersEnd', [], skip=1),
             103: self.wrap(
                 'replaceFAEnd', [int, str], skip=1),
+            104: self.wrap(
+                'wshMetaData', [int, str], skip=1),
+            105: self.wrap(
+                'wshEventData', [int, str], skip=1),
+            106: self.historicalSchedule,
+            107: self.wrap(
+                'userInfo', [int, str], skip=1)
         }
 
     def wrap(self, methodName, types, skip=2):
@@ -213,7 +217,15 @@ class Decoder:
 
         if price:
             self.wrapper.priceSizeTick(
-                int(reqId), int(tickType), float(price), float(size))
+                int(reqId), int(tickType), float(price), float(size or 0))
+
+    def errorMsg(self, fields):
+        _, _, reqId, errorCode, errorString, *fields = fields
+        advancedOrderRejectJson = ''
+        if self.serverVersion >= 166:
+            advancedOrderRejectJson, *fields = fields
+        self.wrapper.error(
+            int(reqId), int(errorCode), errorString, advancedOrderRejectJson)
 
     def updatePortfolio(self, fields):
         c = Contract()
@@ -247,8 +259,10 @@ class Decoder:
     def contractDetails(self, fields):
         cd = ContractDetails()
         cd.contract = c = Contract()
+        if self.serverVersion < 164:
+            fields.pop(0)
         (
-            _, _,
+            _,
             reqId,
             c.symbol,
             c.secType,
@@ -262,7 +276,10 @@ class Decoder:
             c.tradingClass,
             c.conId,
             cd.minTick,
-            cd.mdSizeMultiplier,
+            *fields) = fields
+        if self.serverVersion < 164:
+            fields.pop(0)  # obsolete mdSizeMultiplier
+        (
             c.multiplier,
             cd.orderTypes,
             cd.validExchanges,
@@ -294,21 +311,25 @@ class Decoder:
             cd.underSecType,
             cd.marketRuleIds,
             cd.realExpirationDate,
+            cd.stockType,
             *fields) = fields
-        if self.serverVersion >= 152:
-            cd.stockType, *fields = fields
-        if self.serverVersion >= 163:
-            cd.sizeMinTick, *fields = fields
+        if self.serverVersion == 163:
+            cd.suggestedSizeIncrement, *fields = fields
+        if self.serverVersion >= 164:
+            (
+                cd.minSize,
+                cd.sizeIncrement,
+                cd.suggestedSizeIncrement,
+                # cd.minCashQtySize,
+                *fields) = fields
 
-        times = lastTimes.split()
+        times = lastTimes.split('-' if '-' in lastTimes else None)
         if len(times) > 0:
             c.lastTradeDateOrContractMonth = times[0]
         if len(times) > 1:
             cd.lastTradeTime = times[1]
 
-        if self.serverVersion >= 153:
-            cd.longName = cd.longName.encode().decode('unicode-escape')
-
+        cd.longName = cd.longName.encode().decode('unicode-escape')
         self.parse(cd)
         self.parse(c)
         self.wrapper.contractDetails(int(reqId), cd)
@@ -316,8 +337,10 @@ class Decoder:
     def bondContractDetails(self, fields):
         cd = ContractDetails()
         cd.contract = c = Contract()
+        if self.serverVersion < 164:
+            fields.pop(0)
         (
-            _, _,
+            _,
             reqId,
             c.symbol,
             c.secType,
@@ -338,7 +361,10 @@ class Decoder:
             c.tradingClass,
             c.conId,
             cd.minTick,
-            cd.mdSizeMultiplier,
+            *fields) = fields
+        if self.serverVersion < 164:
+            fields.pop(0)  # obsolete mdSizeMultiplier
+        (
             cd.orderTypes,
             cd.validExchanges,
             cd.nextOptionDate,
@@ -358,9 +384,16 @@ class Decoder:
                 tag, value, *fields = fields
                 cd.secIdList += [TagValue(tag, value)]
 
-        cd.aggGroup, cd.marketRuleIds = fields
+        cd.aggGroup, cd.marketRuleIds, *fields = fields
+        if self.serverVersion >= 164:
+            (
+                cd.minSize,
+                cd.sizeIncrement,
+                cd.suggestedSizeIncrement,
+                # cd.minCashQtySize,
+                *fields) = fields
 
-        times = lastTimes.split()
+        times = lastTimes.split('-' if '-' in lastTimes else None)
         if len(times) > 0:
             cd.maturity = times[0]
         if len(times) > 1:
@@ -485,11 +518,7 @@ class Decoder:
         self.wrapper.scannerDataEnd(int(reqId))
 
     def tickOptionComputation(self, fields):
-        if self.serverVersion >= 156:
-            _, reqId, tickTypeInt, tickAttrib, *fields = fields
-        else:
-            _, _, reqId, tickTypeInt, *fields = fields
-            tickAttrib = 0
+        _, reqId, tickTypeInt, tickAttrib, *fields = fields
         impliedVol, delta, optPrice, pvDividend, \
             gamma, vega, theta, undPrice = fields
 
@@ -788,10 +817,6 @@ class Decoder:
         o = Order()
         c = Contract()
         st = OrderState()
-
-        if self.serverVersion < 145:
-            fields.pop(0)
-
         (
             _,
             o.orderId,
@@ -962,18 +987,12 @@ class Decoder:
             o.solicited,
             o.whatIf,
             st.status,
-            *fields) = fields
-
-        if self.serverVersion >= 142:
-            (
-                st.initMarginBefore,
-                st.maintMarginBefore,
-                st.equityWithLoanBefore,
-                st.initMarginChange,
-                st.maintMarginChange,
-                st.equityWithLoanChange,
-                *fields) = fields
-        (
+            st.initMarginBefore,
+            st.maintMarginBefore,
+            st.equityWithLoanBefore,
+            st.initMarginChange,
+            st.maintMarginChange,
+            st.equityWithLoanChange,
             st.initMarginAfter,
             st.maintMarginAfter,
             st.equityWithLoanAfter,
@@ -1023,22 +1042,26 @@ class Decoder:
             o.softDollarTier.val,
             o.softDollarTier.displayName,
             o.cashQty,
+            o.dontUseAutoPriceForHedge,
+            o.isOmsContainer,
+            o.discretionaryUpToLimitPrice,
+            o.usePriceMgmtAlgo,
             *fields) = fields
 
-        if self.serverVersion >= 141:
-            o.dontUseAutoPriceForHedge = fields.pop(0)
-        if self.serverVersion >= 145:
-            o.isOmsContainer = fields.pop(0)
-        if self.serverVersion >= 148:
-            o.discretionaryUpToLimitPrice = fields.pop(0)
-        if self.serverVersion >= 151:
-            o.usePriceMgmtAlgo = fields.pop(0)
         if self.serverVersion >= 159:
             o.duration = fields.pop(0)
         if self.serverVersion >= 160:
             o.postToAts = fields.pop(0)
         if self.serverVersion >= 162:
             o.autoCancelParent = fields.pop(0)
+        if self.serverVersion >= 170:
+            (
+                o.minTradeQty,
+                o.minCompeteSize,
+                o.competeAgainstBestOffset,
+                o.midOffsetAtWhole,
+                o.midOffsetAtHalf,
+                *fields) = fields
 
         self.parse(c)
         self.parse(o)
@@ -1233,14 +1256,8 @@ class Decoder:
             o.trailStopPrice,
             o.lmtPriceOffset,
             o.cashQty,
-            *fields) = fields
-
-        if self.serverVersion >= 141:
-            o.dontUseAutoPriceForHedge = fields.pop(0)
-        if self.serverVersion >= 145:
-            o.isOmsContainer = fields.pop(0)
-
-        (
+            o.dontUseAutoPriceForHedge,
+            o.isOmsContainer,
             o.autoCancelDate,
             o.filledQuantity,
             o.refFuturesConId,
@@ -1252,7 +1269,31 @@ class Decoder:
             st.completedTime,
             st.completedStatus) = fields
 
+        if self.serverVersion >= 170:
+            (
+                o.minTradeQty,
+                o.minCompeteSize,
+                o.competeAgainstBestOffset,
+                o.midOffsetAtWhole,
+                o.midOffsetAtHalf,
+                *fields) = fields
+
         self.parse(c)
         self.parse(o)
         self.parse(st)
         self.wrapper.completedOrder(c, o, st)
+
+    def historicalSchedule(self, fields):
+        (
+            _,
+            reqId,
+            startDateTime,
+            endDateTime,
+            timeZone,
+            count, *fields) = fields
+        get = iter(fields).__next__
+        sessions = [HistoricalSession(
+            startDateTime=get(), endDateTime=get(), refDate=get())
+            for _ in range(int(count))]
+        self.wrapper.historicalSchedule(
+            int(reqId), startDateTime, endDateTime, timeZone, sessions)
