@@ -14,18 +14,16 @@ from ib_insync.client import Client
 from ib_insync.contract import Contract, ContractDescription, ContractDetails
 from ib_insync.objects import (
     AccountValue, BarDataList, DepthMktDataDescription, Execution,
-    ExecutionFilter, Fill, HistogramData, HistoricalNews, NewsArticle,
-    NewsBulletin, NewsProvider, NewsTick, OptionChain, OptionComputation,
-    PnL, PnLSingle, PortfolioItem, Position, PriceIncrement,
-    RealTimeBarList, ScanDataList, ScannerSubscription, TagValue,
-    TradeLogEntry)
+    ExecutionFilter, Fill, HistogramData, HistoricalNews, HistoricalSchedule,
+    NewsArticle, NewsBulletin, NewsProvider, NewsTick, OptionChain,
+    OptionComputation, PnL, PnLSingle, PortfolioItem, Position, PriceIncrement,
+    RealTimeBarList, ScanDataList, ScannerSubscription, SmartComponent,
+    TagValue, TradeLogEntry)
 from ib_insync.order import (
     BracketOrder, LimitOrder, Order, OrderState, OrderStatus, StopOrder, Trade)
 from ib_insync.ticker import Ticker
 from ib_insync.wrapper import Wrapper
 
-__all__ = ['IB']
-
 
 class IB:
     """
@@ -83,7 +81,7 @@ class IB:
     For introducing a delay, never use time.sleep() but use
     :meth:`.sleep` instead.
 
-    Attributes:
+    Parameters:
         RequestTimeout (float): Timeout (in seconds) to wait for a
           blocking request to finish before raising ``asyncio.TimeoutError``.
           The default value of 0 will wait indefinitely.
@@ -662,14 +660,16 @@ class IB:
             self.newOrderEvent.emit(trade)
         return trade
 
-    def cancelOrder(self, order: Order) -> Trade:
+    def cancelOrder(
+            self, order: Order, manualCancelOrderTime: str = '') -> Trade:
         """
         Cancel the order and return the Trade it belongs to.
 
         Args:
             order: The order to be canceled.
+            manualCancelOrderTime: For audit trail.
         """
-        self.client.cancelOrder(order.orderId)
+        self.client.cancelOrder(order.orderId, manualCancelOrderTime)
         now = datetime.datetime.now(datetime.timezone.utc)
         key = self.wrapper.orderKey(
             order.clientId, order.orderId, order.permId)
@@ -1033,6 +1033,7 @@ class IB:
                 'ADJUSTED_LAST', 'HISTORICAL_VOLATILITY',
                 'OPTION_IMPLIED_VOLATILITY', 'REBATE_RATE', 'FEE_RATE',
                 'YIELD_BID', 'YIELD_ASK', 'YIELD_BID_ASK', 'YIELD_LAST'.
+                For 'SCHEDULE' use :meth:`.reqHistoricalSchedule`.
             useRTH: If True then only show data from within Regular
                 Trading Hours, if False then show all data.
             formatDate: For an intraday request setting to 2 will cause
@@ -1064,6 +1065,29 @@ class IB:
         self.client.cancelHistoricalData(bars.reqId)
         self.wrapper.endSubscription(bars)
 
+    def reqHistoricalSchedule(
+            self, contract: Contract, numDays: int,
+            endDateTime: Union[
+                datetime.datetime, datetime.date, str, None] = '',
+            useRTH: bool = True) -> HistoricalSchedule:
+        """
+        Request historical schedule.
+
+        This method is blocking.
+
+        Args:
+            contract: Contract of interest.
+            numDays: Number of days.
+            endDateTime: Can be set to '' to indicate the current time,
+                or it can be given as a datetime.date or datetime.datetime,
+                or it can be given as a string in 'yyyyMMdd HH:mm:ss' format.
+                If no timezone is given then the TWS login timezone is used.
+            useRTH: If True then show schedule for Regular Trading Hours,
+                if False then for extended hours.
+        """
+        return self._run(self.reqHistoricalScheduleAsync(
+            contract, numDays, endDateTime, useRTH))
+
     def reqHistoricalTicks(
             self, contract: Contract,
             startDateTime: Union[str, datetime.date],
@@ -1205,7 +1229,6 @@ class IB:
         """
         ticker = self.ticker(contract)
         reqId = self.wrapper.endTicker(ticker, 'mktData')
-        self.wrapper.reqId2MarketDataType.pop(reqId, None)
         if reqId:
             self.client.cancelMktData(reqId)
         else:
@@ -1249,6 +1272,15 @@ class IB:
             self._logger.error(
                 f'cancelMktData: No reqId found for contract {contract}')
 
+    def reqSmartComponents(self, bboExchange: str) -> List[SmartComponent]:
+        """
+        Obtain mapping from single letter codes to exchange names.
+
+        Note: The exchanges must be open when using this request, otherwise an
+        empty list is returned.
+        """
+        return self._run(self.reqSmartComponentsAsync(bboExchange))
+
     def reqMktDepthExchanges(self) -> List[DepthMktDataDescription]:
         """
         Get those exchanges that have have multiple market makers
@@ -1614,6 +1646,10 @@ class IB:
         reqId = self.client.getReqId()
         self.client.replaceFA(reqId, faDataType, xml)
 
+    def reqUserInfo(self) -> str:
+        """Get the White Branding ID of the user."""
+        return self._run(self.reqUserInfoAsync())
+
     # now entering the parallel async universe
 
     async def connectAsync(
@@ -1635,9 +1671,11 @@ class IB:
             if not account and len(accounts) == 1:
                 account = accounts[0]
 
-            # prepare initializing  requests
+            # prepare initializing requests
             reqs: Dict = {}  # name -> request
             reqs['positions'] = self.reqPositionsAsync()
+            if not readonly:
+                reqs['open orders'] = self.reqOpenOrdersAsync()
             if not readonly and self.client.serverVersion() >= 150:
                 reqs['completed orders'] = self.reqCompletedOrdersAsync(False)
             if account:
@@ -1678,10 +1716,10 @@ class IB:
         result = []
         for contract, detailsList in zip(contracts, detailsLists):
             if not detailsList:
-                self._logger.error(f'Unknown contract: {contract}')
+                self._logger.warning(f'Unknown contract: {contract}')
             elif len(detailsList) > 1:
                 possibles = [details.contract for details in detailsList]
-                self._logger.error(
+                self._logger.warning(
                     f'Ambiguous contract: {contract}, '
                     f'possibles are {possibles}')
             else:
@@ -1716,8 +1754,6 @@ class IB:
         await asyncio.gather(*futures)
         for ticker in tickers:
             self.wrapper.endTicker(ticker, 'snapshot')
-        for reqId in reqIds:
-            self.wrapper.reqId2MarketDataType.pop(reqId, None)
         return tickers
 
     def whatIfOrderAsync(self, contract: Contract, order: Order) -> \
@@ -1869,6 +1905,19 @@ class IB:
             bars.clear()
         return bars
 
+    def reqHistoricalScheduleAsync(
+            self, contract: Contract, numDays: int,
+            endDateTime: Union[
+                datetime.datetime, datetime.date, str, None] = '',
+            useRTH: bool = True) -> Awaitable[HistoricalSchedule]:
+        reqId = self.client.getReqId()
+        future = self.wrapper.startReq(reqId, contract)
+        end = util.formatIBDatetime(endDateTime)
+        self.client.reqHistoricalData(
+            reqId, contract, end, f'{numDays} D', '1 day', 'SCHEDULE',
+            useRTH, 1, False, None)
+        return future
+
     def reqHistoricalTicksAsync(
             self, contract: Contract,
             startDateTime: Union[str, datetime.date],
@@ -1894,6 +1943,12 @@ class IB:
             reqId, contract, whatToShow, useRTH, formatDate)
         return future
 
+    def reqSmartComponentsAsync(self, bboExchange):
+        reqId = self.client.getReqId()
+        future = self.wrapper.startReq(reqId)
+        self.client.reqSmartComponents(reqId, bboExchange)
+        return future
+
     def reqMktDepthExchangesAsync(self) -> \
             Awaitable[List[DepthMktDataDescription]]:
         future = self.wrapper.startReq('mktDepthExchanges')
@@ -2027,9 +2082,16 @@ class IB:
         except asyncio.TimeoutError:
             self._logger.error('requestFAAsync: Timeout')
 
+    def reqUserInfoAsync(self):
+        reqId = self.client.getReqId()
+        future = self.wrapper.startReq(reqId)
+        self.client.reqUserInfo(reqId)
+        return future
+
 
 if __name__ == '__main__':
-    asyncio.get_event_loop().set_debug(True)
+    loop = util.getLoop()
+    loop.set_debug(True)
     util.logToConsole(logging.DEBUG)
     ib = IB()
     ib.connect('127.0.0.1', 7497, clientId=1)
